#pragma once

#include <list>
#include <string>

#include "Mutex.h"

namespace DetectDriver
{

typedef std::list<std::wstring> FilePathList;

class FileDetectorImpl
{
// Fields synchronized by mutex lock_
    ULONG flag_;
    ULONG status_;

    FilePathList hiddenFilesList_;
    
    std::wstring startPath_;
    std::string errorString_;

    utils::Mutex lock_;

// Fields synchronized by InterlockedXXX
    volatile long isRunning_;
    volatile long totalCount_;
public:
    FileDetectorImpl();

    // User communication
    void Start(const std::wstring& path);
    void Stop();
    void QueryStatus(ULONG* status, ULONG* flag, ULONG* totalCount);
    void QueryHiddenFiles(FilePathList* filePathList);
    void QueryErrorString(std::string* error);

    // Detection callback
    void AddHiddenRecord(const std::wstring& path);
    void OnDirProcessing(const std::wstring& dirPath);
    bool IsStoped()const;
private:
    friend void ThreadWorkerRoutine( void* context );
    void ThreadRoutine();
};

}