#include "drvCommon.h"
#include "DetectDriver_Ioctl.h"

#include "ProcessDetector.h"
#include "PackageParser.h"
#include "ProcNativeApiWork.h"
#include "ProcessListWork.h"
#include "AccessUtils.h"
#include "KernelHandleGuard.h"

#include <string>
#include <stdexcept>

namespace DetectDriver
{
void Detect(ProcInfoShortList* hiddenList)
{
	std::vector<char> hookListBuffer,originalListBuffer;
	GetProcessInfo(&hookListBuffer,&originalListBuffer);

	ProcessInfoList *hookedList = (ProcessInfoList*)&hookListBuffer[0];
	ProcessInfoList *originalList = (ProcessInfoList*)&originalListBuffer[0]; 

	if( IsListsEqual(hookedList,originalList) == false )
	{
		ExamineList(hookedList,originalList,hiddenList);

		if( !hiddenList->empty() )
			return;
	}

	throw std::exception("No hidden process detected.");	
}

// Information presentation
void PackInfo( const ProcInfoFullList& hiddenList, 
			   WCHAR* buf, ULONG bufSize, ULONG *bytesTxd)
{
	if(hiddenList.empty())
		throw std::exception("Empty hidden process list");

	WStringList strHiddenProcList;

	ProcInfoFullList::const_iterator it = hiddenList.begin();
	for( ; it != hiddenList.end() ; ++it )
	{
		wchar_t buff[255];
		std::wstring strPid = _itow((int)it->pid,buff,10);

		std::wstring procRecord;
        utils::PackThreeToOne(strPid,
                              it->name.empty()?L"Null":it->name,
                              it->imagePath.empty()?L"Null":it->imagePath,
                              &procRecord);

		strHiddenProcList.push_back(procRecord);
	}

	std::wstring procListString;
	utils::PackByEOL(strHiddenProcList,&procListString);

	if(procListString.empty())
		throw std::exception("Internal error occur while packing params.");

	if(procListString.size() > bufSize)
		throw std::exception("Buffer size is not enough to hold hidden process list.");

	memcpy(buf,procListString.c_str(),procListString.size()*2);
	*bytesTxd += procListString.size()*2;
}

void RequestMoreInfo(const ProcInfoShortList& hiddenShortInfoList,
					 ProcInfoFullList* hiddenFullInfoList)
{
	ProcInfoShortList::const_iterator it = hiddenShortInfoList.begin();
	for( ; it != hiddenShortInfoList.end() ; ++it )
	{
		std::wstring fileImagePath;
		
		PEPROCESS proc;
		NTSTATUS status = PsLookupProcessByProcessId((PVOID)it->pid,&proc);
		if( NT_SUCCESS(status) )
		{
			utils::ObjectGuard guard(proc);
			utils::GetProcessImagePath(proc,&fileImagePath);
		}
		else
			fileImagePath = L"Can't get image path.";

		ProcessInfoFull fullInfo;
		fullInfo.pid = it->pid;
		fullInfo.name = it->name;
		fullInfo.imagePath = fileImagePath;

		hiddenFullInfoList->push_back(fullInfo);
	}
}

// User communication

void ProcessDetector::Initialize(utils::QueryMng& queryMng)
{
	queryMng.Subscribe(IOCTL_PROCESSES_QUERY_HIDEN,this);
}
void ProcessDetector::Cleanup(utils::QueryMng& queryMng)
{
	queryMng.UnSubscribe(IOCTL_PROCESSES_QUERY_HIDEN,this);
}

void DispatchImpl(WCHAR* buf,ULONG outputBufSize,ULONG* bytesTxd)
{
	ProcInfoShortList hiddenList;
	Detect(&hiddenList);

	ProcInfoFullList hiddenFullInfoList;
	RequestMoreInfo(hiddenList,&hiddenFullInfoList);

	PackInfo(hiddenFullInfoList,buf,outputBufSize,bytesTxd);
}

void ProcessDetector::Dispatch(ULONG controlCode,
							   WCHAR* buf,
							   ULONG inputBufSize,
							   ULONG outputBufSize,
							   ULONG* bytesTxd)
{
	ASSERT(controlCode == IOCTL_PROCESSES_QUERY_HIDEN);

	DbgPrint("-DetectDriver- Query Hidden Processes\n");

	// Need to dispatch exception message by hand
	// to provide information type that will be returned

	// First byte for information type
	char* pDataBuf = (char*)buf + 1;
	char infoType = QUERY_SUCCESS;
	memcpy(buf,&infoType,1);
	*bytesTxd += 1;

	try
	{
		DispatchImpl((wchar_t*)pDataBuf,outputBufSize-1,bytesTxd);
	}
	catch(const std::exception& ex)
	{
		std::string error;
		error += (char)QUERY_FAIL;
		error += ex.what();
		throw std::runtime_error(error);
	}
}

}//namespace DetectDriver