#include "drvCommon.h"

#include "ProcessListWork.h"
#include "ListUtils.h"

#include <algorithm>
#include <boost/bind.hpp>

namespace DetectDriver
{

void GetListSimilarity(ProcessInfoListPtr hookedList,
					   ProcessInfoListPtr originalList,
					   ProcInfoShortList* commonList)
{
	utils::NtList* originalListNt = (utils::NtList*)originalList;
	// Enumerating originalList
	while( true )
	{
		ProcessInfoListPtr originalRecord = (ProcessInfoListPtr)originalListNt;
		utils::NtList* hookedListNt = (utils::NtList*)hookedList;
		// Enumerating hookedList
		while( true )
		{
			ProcessInfoListPtr hookedRecord = (ProcessInfoListPtr)hookedListNt;
			if(RtlEqualUnicodeString(&hookedRecord->ProcessName,
									 &originalRecord->ProcessName, TRUE))
			{
				std::wstring name(originalRecord->ProcessName.Buffer,
								  originalRecord->ProcessName.Length/2);
				ProcessInfoShort shortInfo={originalRecord->ProcessId,name};
				commonList->push_back(shortInfo);
				break;
			}

			if( utils::IsEntryLast(hookedListNt) )
				break;

			hookedListNt = utils::GetNextEntryPointer(hookedListNt);
		}

		if( utils::IsEntryLast(originalListNt))
			break;

		originalListNt = utils::GetNextEntryPointer(originalListNt);
	}
}

void GetListDifference(ProcessInfoListPtr originalListIn,
					   const ProcInfoShortList& commonList,
					   ProcInfoShortList* hiddenList)
{
	utils::NtList* originalListNt = (utils::NtList*)originalListIn;
	// Enumerating hookedList
	while( true )
	{
		ProcessInfoListPtr hookedRecord = (ProcessInfoListPtr)originalListNt;
		std::wstring name(hookedRecord->ProcessName.Buffer,
						  hookedRecord->ProcessName.Length/2);

		ProcInfoShortList::const_iterator it = 
			std::find_if(commonList.begin(),commonList.end(),
						 boost::bind(&ProcessInfoShort::name,_1) == name);
		
		if( it == commonList.end() )
		{
			ProcessInfoShort shortInfo={hookedRecord->ProcessId,name};
			hiddenList->push_back(shortInfo);
		}

		if( utils::IsEntryLast(originalListNt) )
			break;

		originalListNt = utils::GetNextEntryPointer(originalListNt);
	}
}

void ExamineList(ProcessInfoListPtr hookedListIn,
				 ProcessInfoListPtr originalListIn,
				 ProcInfoShortList* hiddenList)
{
	size_t hookedListCount = utils::GetListEntryCount((utils::NtList*)hookedListIn);
	size_t originalListCount = utils::GetListEntryCount((utils::NtList*)originalListIn);
	if( hookedListCount > originalListCount )
		throw std::exception("Hooked list has more records then original.");

	ProcInfoShortList commonList;
	GetListSimilarity(hookedListIn,originalListIn,&commonList);

	if( commonList.size() != originalListCount )
	{
		GetListDifference(originalListIn,commonList,hiddenList);		
	}
}

bool IsListsEqual(ProcessInfoListPtr hookedList,
				  ProcessInfoListPtr originalList)
{
	utils::NtList *ntHookedList = (utils::NtList*)hookedList;
	utils::NtList *ntOriginalList = (utils::NtList*)originalList;

	// Step 1
	size_t hookedListCount = utils::GetListEntryCount(ntHookedList);
	size_t originalListCount = utils::GetListEntryCount(ntOriginalList);

	if(hookedListCount != originalListCount)
		return false;

	// Step 2
	size_t hookedListSize = utils::GetListSize(ntHookedList);
	size_t originalListSize = utils::GetListSize(ntOriginalList);

	if(hookedListSize != originalListSize)
		return false;

	// Step 3
	if(memcmp(hookedList,originalList,hookedListSize) != 0)
		return false;

	return true;
}
}//namespace DetectDriver