#include "drvCommon.h"

#include "WorkItemUtils.h"
#include "KernelEvent.h"

#include <string>
#include <stdexcept>

//The object associated with the driver
extern PDEVICE_OBJECT gDeviceObject;

namespace utils
{

typedef void (*OriginalRoutine)(PVOID);

struct WorkItemParams
{
	OriginalRoutine Routine;
	PVOID params;
	char* strError;
	size_t strErrorSize;
	KernelNotificationEvent* finishEvent;
	NTSTATUS status;
};

VOID WorkItemRoutine(IN PDEVICE_OBJECT DeviceObject,
					 IN PVOID Context)
{
	WorkItemParams* routineParams = (WorkItemParams*)Context;
	routineParams->status = STATUS_SUCCESS;
	
	try
	{
		routineParams->Routine(routineParams->params);
	}
	catch(std::exception& ex)
	{
		routineParams->status = STATUS_UNSUCCESSFUL;
		
		std::string str(ex.what());

		size_t toWrite = min(str.size(),routineParams->strErrorSize - 1);

		memcpy(routineParams->strError,str.c_str(),toWrite);

		routineParams->strError[toWrite] = '\0';
	}

	routineParams->finishEvent->set();
}

void CallFromWorkerThread(PVOID routine,PVOID params)
{
	PIO_WORKITEM workItem = ::IoAllocateWorkItem(gDeviceObject);
	if(workItem == NULL)
		throw std::exception("Can't create work item.");

	utils::WorkItemGuard guard(workItem);

	char errorMsg[255];
	KernelNotificationEvent finishEvent;
	WorkItemParams wiParams = {(OriginalRoutine)routine,params,
							    errorMsg,sizeof(errorMsg),&finishEvent};

	IoQueueWorkItem(workItem,&WorkItemRoutine,DelayedWorkQueue,&wiParams);

	finishEvent.wait();

	if( !NT_SUCCESS(wiParams.status) )
	{
		throw std::runtime_error(errorMsg);
	}	
}

}