#include "drvCommon.h"

#include "FileFind.h"

namespace utils
{
    
void FileFind::Close()
{
    m_pFoundInfo.reset();

    m_pNextInfo.reset();

    m_hContext.reset(NULL);

    m_strRoot = L"";
}

void FileFind::CloseContext()
{
    findApi_.FindClose( m_hContext.get() );
}

BOOL FileFind::FindFile(const std::wstring& path)
{
	Close();

	if( path.empty() )
		return false;
	
    m_pNextInfo.reset( new FileFindBuffer );

	HANDLE hContext = findApi_.FindFirstFile(path.c_str(), m_pNextInfo.get() );

    if( hContext == INVALID_HANDLE_VALUE )
		return FALSE;

    m_hContext.reset(hContext);
    m_strRoot = path;

	return TRUE;
}

BOOL FileFind::FindNextFile()
{
    if (m_hContext.get() == NULL)
        return FALSE;
    if (m_pFoundInfo.get() == NULL)
        m_pFoundInfo.reset( new FileFindBuffer );

    std::swap(m_pFoundInfo,m_pNextInfo);

    return findApi_.FindNextFile( m_hContext.get(), m_pNextInfo.get() );
}

//////////////////////////////  Check  ////////////////////////////// 

void FileFind::CheckHasInfo()const
{
    if( m_hContext.get() == NULL )
        throw std::exception("No context.");

    if( m_pFoundInfo.get() == NULL )
        throw std::exception("No info.");
}

//////////////////////////////  Properties   ////////////////////////////// 

std::wstring FileFind::GetRoot() const
{
    CheckHasInfo();
    
    return m_strRoot;
}

std::wstring FileFind::GetFilePath() const
{
    CheckHasInfo();

    std::wstring strResult = m_strRoot;

    std::wstring::iterator it = strResult.end();
    if( !strResult.empty() )
        if( *(--it) != L'\\' )
            strResult += L'\\';

    strResult += GetFileName();
    return strResult;
}

std::wstring FileFind::GetFileName() const
{
    CheckHasInfo();

    std::wstring fileName;
    fileName.assign(m_pFoundInfo->dirInfo.FileName,
                    m_pFoundInfo->dirInfo.FileName+m_pFoundInfo->dirInfo.FileNameLength/sizeof(wchar_t));
    return fileName;
}

ULONGLONG FileFind::GetEndOfFile() const
{
    CheckHasInfo();

    return m_pFoundInfo->dirInfo.EndOfFile.QuadPart;
}

ULONGLONG FileFind::GetAllocationSize() const
{
    CheckHasInfo();

    return m_pFoundInfo->dirInfo.AllocationSize.QuadPart;
}

DirInfo* FileFind::GetAllInfo()const
{
    return &m_pFoundInfo->dirInfo;
}


BOOL FileFind::IsDots() const
{
    CheckHasInfo();

    BOOL bResult = FALSE;
    if( IsDirectory() )
    {
        const wchar_t* name = m_pFoundInfo->dirInfo.FileName;
        ULONG strSize = m_pFoundInfo->dirInfo.FileNameLength;
        if( name[0] == L'.' )
        {
            if (  strSize == sizeof(wchar_t) 
               || (name[1] == L'.' && strSize == 2*sizeof(wchar_t)) )
            {
                bResult = TRUE;
            }
        }
    }

    return bResult;
}
/*
//////////////////////////////  Time params  ////////////////////////////// 

BOOL FileFind::GetLastAccessTime(FILETIME* pTimeStamp) const
{
    CheckHasInfo();

    if(pTimeStamp != NULL)
        throw std::exception("TimeStamp can't be NULL.");

	if (m_pFoundInfo != NULL && pTimeStamp != NULL)
	{
		*pTimeStamp = ((LPWIN32_FIND_DATA) m_pFoundInfo)->ftLastAccessTime;
		return TRUE;
	}
	else
		return FALSE;
}

BOOL FileFind::GetLastWriteTime(FILETIME* pTimeStamp) const
{
    CheckHasInfo();

    if(pTimeStamp != NULL)
        throw std::exception("TimeStamp can't be NULL.");

	if (m_pFoundInfo != NULL && pTimeStamp != NULL)
	{
		*pTimeStamp = ((LPWIN32_FIND_DATA) m_pFoundInfo)->ftLastWriteTime;
		return TRUE;
	}
	else
		return FALSE;
}

BOOL FileFind::GetCreationTime(FILETIME* pTimeStamp) const
{
    CheckHasInfo();

    if(pTimeStamp != NULL)
        throw std::exception("TimeStamp can't be NULL.");

	if (m_pFoundInfo != NULL && pTimeStamp != NULL)
	{
		*pTimeStamp = ((LPWIN32_FIND_DATA) m_pFoundInfo)->ftCreationTime;
		return TRUE;
	}
	else
		return FALSE;
}
*/

////////////////////////////// Attributes   ////////////////////////////// 

BOOL FileFind::MatchesMask(DWORD dwMask) const
{
    CheckHasInfo();

    return m_pFoundInfo->dirInfo.FileAttributes & dwMask;
}

// Attributes
BOOL FileFind::IsReadOnly() const
{ 
    return MatchesMask(FILE_ATTRIBUTE_READONLY); 
}

BOOL FileFind::IsDirectory() const
{ 
    return MatchesMask(FILE_ATTRIBUTE_DIRECTORY); 
}

BOOL FileFind::IsCompressed() const
{
    return MatchesMask(FILE_ATTRIBUTE_COMPRESSED); 
}

BOOL FileFind::IsSystem() const
{ 
    return MatchesMask(FILE_ATTRIBUTE_SYSTEM); 
}

BOOL FileFind::IsHidden() const
{ 
    return MatchesMask(FILE_ATTRIBUTE_HIDDEN); 
}

BOOL FileFind::IsTemporary() const
{ 
    return MatchesMask(FILE_ATTRIBUTE_TEMPORARY); 
}

BOOL FileFind::IsNormal() const
{ 
    return MatchesMask(FILE_ATTRIBUTE_NORMAL); 
}

BOOL FileFind::IsArchived() const
{ 
    return MatchesMask(FILE_ATTRIBUTE_ARCHIVE); 
}

}