#pragma once

#include <string>
#include <memory>

#include "KernelHandleGuard.h"
#include "FileFindApi.h"

namespace utils
{
    struct FileFindBuffer;
}

namespace utils
{

class FileFind
{
    typedef std::auto_ptr<FileFindBuffer> FileFindBufPtr;
    FileFindBufPtr m_pFoundInfo;
    FileFindBufPtr m_pNextInfo;
    utils::HandleGuard m_hContext;
    std::wstring m_strRoot;

    FileFindApi findApi_;
public:
    FileFind(IEnvFileApi* envApi)
        : findApi_(envApi)
    {}

    // Properties    
	ULONGLONG GetEndOfFile() const;
    ULONGLONG GetAllocationSize() const;
    std::wstring GetFileName() const;
	std::wstring GetFilePath() const;
	std::wstring GetRoot() const;
    BOOL IsDots() const;
    DirInfo* GetAllInfo()const;

/*
    // Time params
	BOOL GetLastWriteTime(FILETIME* pTimeStamp) const;
	BOOL GetLastAccessTime(FILETIME* pTimeStamp) const;
	BOOL GetCreationTime(FILETIME* pTimeStamp) const;
*/
    // Attributes
	BOOL MatchesMask(DWORD dwMask) const;

	BOOL IsReadOnly() const;
	BOOL IsDirectory() const;
	BOOL IsCompressed() const;
	BOOL IsSystem() const;
	BOOL IsHidden() const;
	BOOL IsTemporary() const;
	BOOL IsNormal() const;
	BOOL IsArchived() const;


// Operations
	void Close();
    BOOL FindFile(const std::wstring& path);
	BOOL FindNextFile();

private:
	void CloseContext();
    void CheckHasInfo()const;
};

}