#pragma once

#include "drvCommon.h"
#include "HandleGuardImpl.h"
#include "boost/noncopyable.hpp"

namespace utils
{
/************************************************************************/
/* CWriteReadSection class used for threads synchronization.                       */
/************************************************************************/
class WriteReadSection 
{
public:
    WriteReadSection();
    ~WriteReadSection();

    VOID WaitToRead();        // Call this to gain shared read access
    VOID WaitToWrite();       // Call this to gain exclusive write access
    VOID Done();              // Call this when done accessing the resource

private:
    KSPIN_LOCK    m_SpinLock;    // Permits exclusive access to other members
    KSEMAPHORE    m_hsemReaders; // Readers wait on this if a writer has access
    KSEMAPHORE    m_hsemWriters; // Writers wait on this if a reader has access
    int    m_nWaitingReaders;    // Number of readers waiting for access
    int    m_nWaitingWriters;    // Number of writers waiting for access
    int    m_nActive;            // Number of threads currently with access
                                //   (0=no threads, >0=# of readers, -1=1 writer)
};

class SectionDoneAction
{
public:
	static void Execute(WriteReadSection* section)
	{
        section->Done();
	}
};
typedef HandleGuardImpl<WriteReadSection*,SectionDoneAction> SectionGuard;

class SectionReadGuard:public SectionGuard
{
public:
	SectionReadGuard(WriteReadSection* section)
		: SectionGuard(section)
	{
		section->WaitToRead();
	}
};

class SectionWriteGuard:public SectionGuard
{
public:
	SectionWriteGuard(WriteReadSection* section)
		: SectionGuard(section)
	{
		section->WaitToWrite();
	}
};

}//namespace utils